
import martian
import grokcore.component
import grokcore.view

from zeam.form.viewlet.form import ViewletManagerForm


class ManagerGrokker(martian.ClassGrokker):
    martian.component(ViewletManagerForm)
    martian.directive(grokcore.component.name,
                      get_default=grokcore.view.meta.views.default_view_name)

    def execute(self, factory, config, name, **kw):
        if factory.prefix is None:
            factory.prefix = name
            return True
        return False



"""

We define a viewlet form here.

Let's grok our example:

  >>> from zeam.form.viewlet.testing import grok
  >>> grok('zeam.form.viewlet.ftests.forms.viewlet')

Let's render the view that use our viewlet now:

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.viewlet.ftests.forms.viewlet import Context
  >>> context = Context()

Integration tests
-----------------

  >>> root = getRootFolder()
  >>> root['content'] = context

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

Submission
~~~~~~~~~~

We are going just to submit the form without giving any required
information, and we should get an error:

  >>> browser.open('http://localhost/content/index')
  >>> print browser.contents
  <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
  <div>
    <form action="http://localhost/content/index" method="post" enctype="multipart/form-data">
      <h3>Subscription corner</h3>
      <div class="fields">
        <div class="field">
          <label class="field-label" for="form-field-name">Name</label>
          <br />
          <input type="text" id="form-field-name" name="form.field.name" class="field" value="" />
        </div> <div class="field">
          <label class="field-label" for="form-field-email">Email</label>
          <br />
          <input type="text" id="form-field-email" name="form.field.email" class="field" value="" />
        </div>
      </div>
      <div class="actions">
        <div class="action">
          <input type="submit" id="form-action-subscribe" name="form.action.subscribe" value="Subscribe" class="action" />
        </div>
      </div>
    </form>
  </div>

  >>> 'Registration done for Sylvain' not in browser.contents
  True
  >>> browser.getControl('Name').value = "Sylvain"
  >>> browser.getControl('Email').value = "sylvain at example dot com"
  >>> browser.getControl('Subscribe').click()
  >>> 'Registration done for Sylvain' in browser.contents
  True

"""

from zeam.form import viewlet as zeam
from grokcore import viewlet as grok


class Context(grok.Context):
    pass


class Index(grok.View):
    grok.context(Context)


class Manager(grok.ViewletManager):
    grok.context(Context)
    grok.view(Index)
    grok.name('manager')


class Form(zeam.ViewletForm):
    grok.context(Context)
    grok.view(Index)
    grok.viewletmanager(Manager)

    label = 'Subscription corner'
    fields = zeam.Fields(zeam.Field('Name'), zeam.Field('Email'))

    @zeam.action("Subscribe")
    def subscribe(self):
        data, errors = self.extractData()
        if errors:
            return zeam.FAILURE
        self.status = "Registration done for %s" % (data['name'])
        return zeam.SUCCESS




from zeam.form.base.form import FormCanvas
from zeam.form.viewlet.interfaces import IInlineForm

from megrok import pagetemplate as pt
from grokcore import viewlet as grok

pt.templatedir('default_templates')


class ViewletManagerForm(grok.ViewletManager, FormCanvas):
    grok.baseclass()
    grok.implements(IInlineForm)
    i18nLanguage = None
    prefix = None

    def __init__(self, context, request, view):
        grok.ViewletManager.__init__(self, context, request, view)
        FormCanvas.__init__(self, context, request)

    def update(self):
        # We are interested by the viewlet manager feature
        # here. should we call super ?
        # grok.ViewletManager.update(self)
        FormCanvas.update(self)
        self.updateForm()

    def default_namespace(self):
        namespace = super(ViewletManagerForm, self).default_namespace()
        namespace['form'] = self
        if self.i18nLanguage is not None:
            namespace['target_language'] = self.i18nLanguage
        return namespace

    def updateForm(self):
        self.updateActions()
        self.updateWidgets()

    def render(self):
        return FormCanvas.render(self)

    render.base_method = True


class ViewletForm(grok.Viewlet, FormCanvas):
    """A form as a viewlet.
    """
    grok.baseclass()
    grok.implements(IInlineForm)
    i18nLanguage = None

    def __init__(self, context, request, view, manager):
        grok.Viewlet.__init__(self, context, request, view, manager)
        FormCanvas.__init__(self, context, request)

    def update(self):
        grok.Viewlet.update(self)
        FormCanvas.update(self)
        self.updateForm()

    def default_namespace(self):
        namespace = super(ViewletForm, self).default_namespace()
        namespace['form'] = self
        if self.i18nLanguage is not None:
            namespace['target_language'] = self.i18nLanguage
        return namespace

    def updateForm(self):
        self.updateActions()
        self.updateWidgets()

    def render(self):
        return FormCanvas.render(self)

    render.base_method = True


class FormTemplate(pt.PageTemplate):
    """Default tempalte for ViewletForm
    """
    pt.view(IInlineForm)



# -*- coding: utf-8 -*-

import os.path
import re
import zeam.form.viewlet

from zope.testing import renormalizing
from zope.app.testing.functional import ZCMLLayer, FunctionalTestSetup
from zope.configuration.config import ConfigurationMachine
from grokcore.component import zcml

ftesting_zcml = os.path.join(
    os.path.dirname(zeam.form.viewlet.__file__), 'ftesting.zcml')
FunctionalLayer = ZCMLLayer(
    ftesting_zcml, __name__, 'FunctionalLayer')


def setUp(test):
    FunctionalTestSetup().setUp()


def tearDown(test):
    FunctionalTestSetup().tearDown()


def grok(module_name):
    config = ConfigurationMachine()
    zcml.do_grok(module_name, config)
    config.execute_actions()


checker = renormalizing.RENormalizing([
    # str(Exception) has changed from Python 2.4 to 2.5 (due to
    # Exception now being a new-style class).  This changes the way
    # exceptions appear in traceback printouts.
    (re.compile(r"ConfigurationExecutionError: <class '([\w.]+)'>:"),
                r'ConfigurationExecutionError: \1:')])



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$


from zeam.form.base.interfaces import IFormCanvas


class IInlineForm(IFormCanvas):
    """A form that can be inserted inside an another page / layout.
    """



from setuptools import setup, find_packages
import os

version = '1.1'

tests_require = [
    'zope.app.authentication',
    'zope.app.testing',
    'zope.app.zcmlfiles',
    'zope.configuration',
    'zope.publisher',
    'zope.securitypolicy',
    'zope.testbrowser',
    'zope.testing',
    ]

setup(name='zeam.form.viewlet',
      version=version,
      description="zeam.form forms in a viewlet",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='zeam form viewlet',
      author='Sylvain Viollon',
      author_email='thefunny@gmail.com',
      url='http://pypi.python.org/pypi/zeam.form.viewlet',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['zeam', 'zeam.form'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'grokcore.component',
        'grokcore.view',
        'grokcore.viewlet',
        'martian',
        'megrok.pagetemplate',
        'setuptools',
        'zeam.form.base >= 1.0',
        ],
      tests_require = tests_require,
      extras_require = {'test': tests_require},
      )
